package com.mokabyte.mokabook.jmx;

import javax.management.*;
import java.util.*;

/**
 * Applicazione client che mostra il meccanismo di invocazione diretta per oggetti e Server Agent JMX in esecuzione
 * all'interno della stessa JVM.
 * Crea un server Agent JMX, vi registra un oggetto User, e ne invoca il metodo printInfo()
 *
 * La stampa di messaggi sulla console serve per seguire il comportamento della applicazione: sostituire con un
 * sistema di log come log4j o altro.
 */


public class UserClient {

  // il costruttore pubblico e vuoto rispetta le specifiche Java Beans
  public UserClient() {}

  public void run() throws Exception{
    System.out.println("--- Enter run()");
    // cerca un server Agent dalla JVM, ricavando la lista
    // di tutti i server in esecuzione
    List ServerList = MBeanServerFactory.findMBeanServer(null);
    MBeanServer Server = (MBeanServer)ServerList.iterator().next();
    System.out.println("Trovato in esecuzione un server Agent; MBeanServer ="+Server );

    ObjectName Name = new ObjectName("example:name=user");
    Server.registerMBean(new User(), Name);
    Object result = Server.invoke(Name,"printInfo",null, null);
    System.out.println("Esecuzione JMX senza inizializzazione dell'oggetto - risultato: \n"+result);
    System.out.println("\n -------------\n");


    // Inizializzazione dell'MBean con propriet
    System.out.println("Inizializzazione di propriet");
    AttributeList Attributes = new AttributeList();
    Attributes.add(new Attribute("Name","Giovanni Puliti"));
    Attributes.add(new Attribute("Address","piazza Moka 11"));
    Attributes.add(new Attribute("Password","pippo"));

    Vector Numbers = new Vector();
    Numbers.add("055-12233455");
    Numbers.add("055-1223345566");
    Numbers.add("055-1223345577");
    Numbers.add("055-1223345588");
    Attributes.add(new Attribute("PhoneNumbers", Numbers));
    Server.setAttributes(Name, Attributes);
    result = Server.invoke(Name, "printInfo", null, null);
    System.out.println("Esecuzione JMX dopo inizializzazione dell'oggetto - risultato: \n" + result);

    System.out.println("--- Exit run()");
  }

  public static void main(String[] args) throws Exception {
    // Crea un server JMX che in questo esempio rappresente
    // l'unico server in esecuzione in questa JVM
    // il metodo findMBeanServer() invocato trover quest'unica istanza
    // in esecuzione
    MBeanServer server = MBeanServerFactory.createMBeanServer();
    UserClient userClient1 = new UserClient();
    userClient1.run();
  }
}