package com.mokabyte.mokabook.jmx;

import javax.management.*;
import java.util.List;
import javax.management.MBeanRegistrationException;
import javax.management.InstanceAlreadyExistsException;

import com.mokabyte.mokabook.jmx.UserListener;
import javax.management.MBeanException;
import javax.management.InvalidAttributeValueException;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;


/**
 * Applicazione client che crea e registra un Agent Server sul quale poi vengono
 * registrati tre oggetti BroadcastingUser, sui quali poi viene invocato metodo remove
 * che provvede a lanciare un evento di notifica verso gli ascoltatori UserListener.
 * Ipotizzando il caso di una ipotetica  applicazione di gestione degli utenti di una community,
 * i soli messaggi inoltrati agli ascoltatori sono quelli filtrati dal filtro RemoveUserFilter
 * il quale controlla che il messaggio sia di tipo com.mokabyte.community.remove
 *
 * La stampa di messaggi sulla console serve per seguire il comportamento della applicazione: sostituire con un
 * sistema di log come log4j o altro.
 */
public class ListenerClient {

  private NotificationListener NListener=null;
  private NotificationFilter NFilter = null;
  private MBeanServer Server = null;

  public void run() {
    // cerca tutte le istanza di Agent server in esecuzione nella stessa JVM
    List ServerList = MBeanServerFactory.findMBeanServer(null);
    Server = (MBeanServer)ServerList.iterator().next();

    NListener = new UserListener(Server);
    NFilter = new RemoveUserFilter();

    ObjectName Giovanni = null, Andrea = null, Franco = null;
    // crea tre nomi da usare come ObjectName per i tre User che verranno
    // registrati nel server
    try {
      Giovanni = new ObjectName("user:name=Giovanni");
      Andrea = new ObjectName("user:name=Andrea");
      Franco = new ObjectName("user:name=Franco");
    }
    catch (MalformedObjectNameException ex) {
      System.out.println("MalformedObjectNameException "+ex);
    }

    // registra tre User con i nomi precedentemente creati
    try {
      Server.registerMBean(new BroadcastingUser(), Giovanni);
      Server.registerMBean(new BroadcastingUser(), Andrea);
      Server.registerMBean(new BroadcastingUser(), Franco);
    }
    catch (NotCompliantMBeanException ex) {
      System.out.println("NotCompliantMBeanException "+ex);
    }
    catch (MBeanRegistrationException ex) {
      System.out.println("MBeanRegistrationException "+ex);
    }
    catch (InstanceAlreadyExistsException ex) {
      System.out.println("InstanceAlreadyExistsException "+ex);
    }

    // setta gli attributi agli utenti
    try {
      System.out.println("Setta gli attributi Name per i tre utenti");
      Server.setAttribute(Giovanni, new Attribute("Name", "Giovanni"));
      Server.setAttribute(Andrea, new Attribute("Name","Andrea"));
      Server.setAttribute(Franco, new Attribute("Name","Franco"));
    }
    catch (ReflectionException ex) {
      System.out.println("ReflectionException "+ex);
    }
    catch (MBeanException ex) {
      System.out.println("MBeanException "+ex);
    }
    catch (InvalidAttributeValueException ex) {
      System.out.println("InvalidAttributeValueException "+ex);
    }
    catch (AttributeNotFoundException ex) {
      System.out.println("AttributeNotFoundException "+ex);
    }
    catch (InstanceNotFoundException ex) {
      System.out.println("InstanceNotFoundException "+ex);
    }

    // aggiunge i listener al server. Si noti che la registrazione   associata
    // ad un determinato evento solo tramite il filter
    try {
      System.out.println("Aggiunge il listener ai tre oggetti user passando un filtro");
      Server.addNotificationListener(Giovanni, NListener, NFilter, Giovanni);
      Server.addNotificationListener(Andrea, NListener, NFilter, Andrea);
      Server.addNotificationListener(Franco, NListener, NFilter, Franco);
    }
    catch (InstanceNotFoundException ex) {
      System.out.println("InstanceNotFoundException "+ex);
    }

    // invoca i metodi di business remove sui tre oggetti registrati presso il server
    try {
      Server.invoke(Giovanni, "remove", null, null);
      Server.invoke(Andrea,"remove",null,null);
      Server.invoke(Franco,"remove",null,null);
    }
    catch (ReflectionException ex) {
      System.out.println("ReflectionException "+ex);
    }
    catch (MBeanException ex) {
      System.out.println("MBeanException "+ex);
    }
    catch (InstanceNotFoundException ex) {
      System.out.println("InstanceNotFoundException "+ex);
    }
  }

  public static void main(String[] args) {
    // Crea un server JMX che in questo esempio rappresente
   // l'unico server in esecuzione in questa JVM
   // il metodo findMBeanServer() invocato trover quest'unica istanza
   // in esecuzione
   MBeanServer server = MBeanServerFactory.createMBeanServer();

   ListenerClient listenerClient1 = new ListenerClient();
   listenerClient1.run();
  }

}